<!doctype html>
<html>
  <head>
  <title>DNA Center Ansible Actions documentation index</title>
  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
  <link rel="stylesheet" media="all" href="https://theforeman.org/static/css/bootstrap.min.css"/>
  <link rel="stylesheet" media="all" href="https://theforeman.org/static/css/bootstrap-responsive.min.css"/>
  <link rel="stylesheet" media="all" href="https://theforeman.org/static/css/style.css"/>
  <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/font-awesome/4.3.0/css/font-awesome.min.css">
  </head>

  <body style="margin:0px 15px">
    <h1>DNA Center Ansible Actions</h1>

    <hr />

    <div class="row plugin-manual">
